#========================================================================================
# package      DE_EPAGES::Order::API::Resource::CartCreationResource
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for Cart/basket Creation
#========================================================================================
package DE_EPAGES::Order::API::Resource::CartCreationResource;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::Formatter;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Order::API::BasketCreator;
use DE_EPAGES::Order::API::Resource::Cart;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError Error );
use feature 'switch';

use strict;



sub service {
    return LoadObjectByPath('/RestServices/CartCreationResource');
}

sub class {
    return 'Shop';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        POST => [
            ['post', 'application/vnd.epages.v1+json'],
            ['post', 'application/json'],
        ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/carts/?$|;
    return 1;
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;
    my $SellingCountryID = $Request->header('X-EPAGES-SELLINGCOUNTRYID');
    return [ $Object, $SellingCountryID ];
}

sub post {
    my $self = shift;
    my ($Shop, $SellingCountryID, $hRequestBody) = @_;

    $hRequestBody->{'sellingCountryId'} = $SellingCountryID if defined $SellingCountryID;

    #-- If delivery thresholds is active, we NEED a selling country ID. But the API doesn't allow for this field.
    #-- So just use the default shop country
    #-- Once a billing/shipping address is added, this field is no longer required/used
    if ( ! defined $hRequestBody->{'sellingCountryId'} && $Shop->get('DeliveryThresholdIsActive') ) {
        foreach my $hInfo ( @{$Shop->get('ShopCountries')} ) {
            if ( $hInfo->{'IsDefault'} ) {
                $hRequestBody->{'sellingCountryId'} = $hInfo->{'CountryID'};
                $SellingCountryID = $hInfo->{'CountryID'};
            }
        }
    }

    my $BasketCreator = DE_EPAGES::Order::API::BasketCreator->new($Shop);
    my $Basket;
    my $hErrors = {};
    eval{
        $Basket = $BasketCreator->createBasket($hRequestBody, $hErrors);
    };

    if (ExistsError()) {
        my $Error = GetError();
        if($Error->code() eq 'Low StockLevel'){
            Error('BadRequest', {'Reason' => 'StockLevel is lower then requested quantity for one or more products'});
        }
        elsif($Error->code() eq 'ProductNoPrice'){
            Error('BadRequest', {'Reason' => 'Price is missing for one or more products'});
        }
        Error($Error);
    }

    my $CartResource = DE_EPAGES::Order::API::Resource::Cart->new();
    my $ReturnValue = $CartResource->get($Basket, [], $SellingCountryID);
    $ReturnValue->{'status'} = $hErrors->{'status'} if exists $hErrors->{'status'};

    return $ReturnValue;
}

1;
