#==================================================================================================
# package      DE_EPAGES::Paybox::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object;
# state        private
#==================================================================================================
package DE_EPAGES::Paybox::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use DE_EPAGES::Core::API::Log qw ( LogCommunication );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Paybox::API::GatewayInterface::PaymentGatewayFactory qw (
    LoadPaymentGatewayConfigurator
    LoadPaymentGateway
);

#==================================================================================================
# ViewMultiPaymentPaybox event handler
#==================================================================================================
sub ViewMultiPaymentPaybox {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    $self->ViewPaymentPaybox($Servlet);

    return;
}

#==================================================================================================
# ViewPaymentPaybox event handler
#==================================================================================================
sub ViewPaymentPaybox {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Object  = $Servlet->object;    # may be a basket or a customerorder
    my $LineItemPayment = $Object->get('LineItemContainer')->get('Payment');
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');

    $LineItemPayment->set( {
            'IsTestTransaction' => $PaymentMethod->get('IsInTestMode'),
            'TransID' => $LineItemPayment->get('GUID'),
            'TransTime' => GetCurrentDateTime(),
            'PaymentErrorCode' => undef,
         } );

    my $PaymentGatewayConfigurator = LoadPaymentGatewayConfigurator($PaymentMethod->id);
    my $PaymentGateway = LoadPaymentGateway($PaymentGatewayConfigurator);
    my $RequestURL = $PaymentGateway->getPaymentRequestURL();
    my $aRequestData = $PaymentGateway->getPaymentRequestData($LineItemPayment);

    LogCommunication('Request parameters', $aRequestData, 'Paybox');

    $Servlet->vars('PayboxRedirectURL', $RequestURL);
    $Servlet->vars('PayboxRedirectData', $aRequestData);
    $Servlet->vars('IsTestTransaction', $LineItemPayment->get('IsTestTransaction'));

    return;
}

1;
