#========================================================================================
# package      DE_EPAGES::Paybox::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Paybox::UI::UI_Shop
#========================================================================================
package DE_EPAGES::Paybox::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Number Object String );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall WasCalled );
use Test::More;

use strict;

use constant SHOP_NAME => 'MyTestShop';
use constant ORDER_ID => 'MyOrderId';
use constant BASKET_ID => 'MyBasketId';

sub test {
    my $self = shift;

    $self->test_CallbackPayment_Successful();
    $self->test_CallbackPayment_Error();
    $self->test_BacklinkPayment_OrderBeforePayment_Successful_UnityShop();
    $self->test_BacklinkPayment_OrderAfterPayment_Successful_UnityShop();
    $self->test_BacklinkPayment_OrderBeforePayment_Error_UnityShop();
    $self->test_BacklinkPayment_OrderAfterPayment_Error_UnityShop();
    $self->test_BacklinkPayment_OrderBeforePayment_ErrorCode_BaseShop();
    $self->test_BacklinkPayment_OrderAfterPayment_ErrorCode_BaseShop();

    done_testing();
}

sub test_CallbackPayment_Successful {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithSuccessfulResponse();
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->CallbackPayment($MockedServlet);

    my $hProcessPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'processPaymentResponse')->[0];
    is_deeply($hProcessPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - processPaymentResponse set Shop parameter ok.");
    isa_ok($hProcessPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - processPaymentResponse set CallbackParams parameter ok.");

    my $aWriteContentOutputParams = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput');
    is($aWriteContentOutputParams->[0], 'OK', "$SubTestName - servlet write content output is ok.")
}

sub test_CallbackPayment_Error {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShop = $self->_createShop();
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithErrorResponse();
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->CallbackPayment($MockedServlet);

    my $hProcessPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'processPaymentResponse')->[0];
    is_deeply($hProcessPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - processPaymentResponse set Shop parameter ok.");
    isa_ok($hProcessPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - processPaymentResponse set CallbackParams parameter ok.");

    my $aWriteContentOutputParams = GetParametersFromMockifyCall($MockedServlet, 'writeContentOutput');
    is($aWriteContentOutputParams->[0], 'ERROR', "$SubTestName - servlet write content output is error.")
}

sub test_BacklinkPayment_OrderBeforePayment_Successful_UnityShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 1;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithSuccessfulResponse($self->_createOrder());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    is($MockedServlet->vars('ViewObjectID'), ORDER_ID, "$SubTestName - servlet ViewObjectID variable set with the Order ID.");
    is($MockedServlet->vars('ViewAction'), 'ViewMultiCheckoutCustomerOrder', "$SubTestName - servlet ViewAction variable set correctly.");
}

sub test_BacklinkPayment_OrderAfterPayment_Successful_UnityShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 1;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithSuccessfulResponse($self->_createBasket());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    is($MockedServlet->vars('ViewObjectID'), BASKET_ID, "$SubTestName - servlet ViewObjectID variable set with the Basket ID.");
    is($MockedServlet->vars('ViewAction'), 'ViewMultiCheckoutCustomerOrder', "$SubTestName - servlet ViewAction variable set correctly.");
}

sub test_BacklinkPayment_OrderBeforePayment_Error_UnityShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 1;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithErrorResponse($self->_createOrder());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    is($MockedServlet->vars('ViewObjectID'), ORDER_ID, "$SubTestName - servlet ViewObjectID variable set with the Order ID.");
    is($MockedServlet->vars('ViewAction'), 'ViewMultiCheckoutCustomerOrder', "$SubTestName - servlet ViewAction variable set correctly.");
}

sub test_BacklinkPayment_OrderAfterPayment_Error_UnityShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 1;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedServlet = $self->_createServlet($MockedShop);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithErrorResponse($self->_createBasket());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    is($MockedServlet->vars('ViewObjectID'), BASKET_ID, "$SubTestName - servlet ViewObjectID variable set with the Basket ID.");
    is($MockedServlet->vars('PayboxPaymentFailed'), 1, "$SubTestName - servlet PayboxPaymentFailed variable set to 1.");
    ok(WasCalled($SUT, '_redirectToPaymentStep'), "$SubTestName - redirection to payment step applied.");
}

sub test_BacklinkPayment_OrderBeforePayment_ErrorCode_BaseShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 0;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithErrorResponse($self->_createOrder());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    ok(WasCalled($MockedForm, 'executeFormError'), "$SubTestName - redirection in base shop through form error.");
}

sub test_BacklinkPayment_OrderAfterPayment_ErrorCode_BaseShop {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $IsUnityShop = 0;
    my $MockedShop = $self->_createShop($IsUnityShop);
    my $MockedForm = $self->_createForm();
    my $MockedServlet = $self->_createServlet($MockedShop, $MockedForm);
    my $MockedPaymentGatewayConfigurator = $self->_createPaymentGatewayConfigurator();
    my $MockedPaymentGateway = $self->_createPaymentGatewayWithErrorResponse($self->_createBasket());
    my $SUT = $self->_createSUT($MockedPaymentGatewayConfigurator, $MockedPaymentGateway);

    $SUT->BacklinkPayment($MockedServlet);

    my $hCheckPaymentResponseParams = GetParametersFromMockifyCall($MockedPaymentGateway, 'checkPaymentResponse')->[0];
    is_deeply($hCheckPaymentResponseParams->{'Shop'}->alias(), SHOP_NAME, "$SubTestName - checkPaymentResponse set Shop parameter ok.");
    isa_ok($hCheckPaymentResponseParams->{'CallbackParams'}, 'DE_EPAGES::Paybox::API::PaymentCallbackParams', "$SubTestName - checkPaymentResponse set CallbackParams parameter ok.");

    ok(WasCalled($MockedForm, 'executeFormError'), "$SubTestName - redirection in base shop through form error.");
}

#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($MockedPaymentGatewayConfigurator, $MockedPaymentGateway) = @_;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::Paybox::UI::Shop', $aParameterList);

    $MockifySut->mockImported('DE_EPAGES::Core::API::Log', 'GetLog')
            ->whenAny()->thenReturn($self->_createLogger());
    $MockifySut->mockImported('DE_EPAGES::Core::API::Log', 'LogCommunication')
            ->whenAny()->thenReturnUndef();

    $MockifySut->mockImported('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGatewayFactory', 'LoadPaymentGatewayConfigurator')
            ->when(Number())->thenReturn($MockedPaymentGatewayConfigurator);
    $MockifySut->mockImported('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGatewayFactory', 'LoadPaymentGateway')
            ->when(Object('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGatewayConfigurator'))
            ->thenReturn($MockedPaymentGateway);
    $MockifySut->mockImported('DE_EPAGES::Paybox::API::PaymentDispatcher', 'LoadPaymentDispatcher')
            ->when()->thenReturnUndef();

    $MockifySut->mock('BlockMonitor')->when(Object('DE_EPAGES::Presentation::Servlet::Permission'))->thenReturnUndef();
    $MockifySut->mock('_redirectToPaymentStep')->whenAny()->thenReturnUndef();

    return $MockifySut->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($IsUnityShop) = @_;

    my $Attributes = {
        'Alias' => SHOP_NAME,
        'IsUnityRegistered' => $IsUnityShop // 1,
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $Attributes);

    return $MockifyShop->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ( $Shop, $Form ) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    my $hServletParams = {
        'Amount' => '2990',
        'PaymentId_TransId' => '22735_6479C12A-32C1-520F-A1FE-D580924525F2_1685700911722.98'
    };
    $MockifyServlet->addParams($hServletParams);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', 1, ['string']);
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('registerSession', undef, ['any', 'int']);
    if ($Form) {
        $MockifyServlet->addMockWithReturnValue( 'form', $Form );
    }

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Presentation::API::Form', []);
    $Mockify->mock('executeFormError', undef, ['hashref']);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPaymentGatewayConfigurator {
    my $self = shift;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGatewayConfigurator');

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPaymentGatewayWithSuccessfulResponse {
    my $self = shift;
    my ($BasketOrder) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGateway');
    $Mockify->mock('processPaymentResponse')->when(HashRef())->thenReturn(1);

    if ($BasketOrder) {
        $Mockify->mock('checkPaymentResponse')->when(HashRef())->thenReturn( { 'Order' => $BasketOrder } );
    }

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPaymentGatewayWithErrorResponse {
    my $self = shift;
    my ($BasketOrder) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::Paybox::API::GatewayInterface::PaymentGateway');
    $Mockify->mock('processPaymentResponse')->when(HashRef())->thenThrowError('payment_process_failed');

    if ($BasketOrder) {
        $Mockify->mock('checkPaymentResponse')
                ->when(HashRef())
                ->thenReturn( { 'ErrorCode' => 'some-error', 'Order' => $BasketOrder } );
    } else {
        $Mockify->mock('checkPaymentResponse')->when(HashRef())->thenThrowError('check_process_failed');
    }

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $Attributes = {
        'ID' => ORDER_ID,
        'instanceOf' => 'CustomerOrder',
        'User'  => undef,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder',
            $Attributes
    );
    $Mockify->addMockWithReturnValue('class', $self->_createClass('CustomerOrder'));

    return $Mockify->getMockObject();;
}

#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;

    my $Attributes = {
        'ID' => BASKET_ID,
        'instanceOf' => 'Basket',
        'User'  => undef,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::Basket',
            $Attributes
    );
    $Mockify->addMockWithReturnValue('class', $self->_createClass('Basket'));

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createLogger {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'Log::Log4perl::Logger' );
    $Mockify->mock('debug')->when(String())->thenReturn('helloWorld');

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createClass {
    my $self = shift;
    my ($ClassName) = @_;

    my $Attributes = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Class',
            $Attributes
    );
    $Mockify->addMockWithReturnValue('alias', $ClassName);

    return $Mockify->getMockObject();
}

#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
