#========================================================================================
# package      DE_EPAGES::Product::API::ProductTypePatch
# state        public
#----------------------------------------------------------------------------------------
# description  Implements JSON-PATCH (RFC 6902) for the product object.
#========================================================================================
package DE_EPAGES::Product::API::ProductTypePatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw (Error);
use strict;

## no critic (ProhibitUnusedPrivateSubroutines)

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $ProductTypeService, $ProductType, $LocaleID );
#----------------------------------------------------------------------------------------
# description  Constructor for ProductTypePatch
#----------------------------------------------------------------------------------------
# input        $ProductTypeService | A ProductTypeService | DE_EPAGES::Product::API::ProductTypeService
# input        $ProductType | A ProductType | DE_EPAGES::Product::API::Object::ProductType
# input        $LocaleID | A LocaleID | integer
# input        $CurrencyId | A CurrencyId | integer
# return       $self | self | DE_EPAGES::Product::API::ProductTypePatch
#========================================================================================
sub new {
    my $class = shift;
    my ($ProductTypeService, $ProductType, $LocaleID, $CurrencyId) = @_;
    my $self  = bless {
        'ProductTypeService' => $ProductTypeService,
        'ProductType' => $ProductType,
        'LocaleId' => $LocaleID,
        'CurrencyId' => $CurrencyId,
    }, $class;
    return $self;
}

sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $ProductTypeService = $self->{'ProductTypeService'};
    my $ProductType = $self->{'ProductType'};
    my $LocaleID = $self->{'LocaleId'};

    if($Path eq '/attributevalues') {
        # add new attribute value for the product type attribute
        my $hRequestBody->{'attributes'} = [$Value];
        $ProductTypeService->createProductTypeAttributes($ProductType, $LocaleID, $hRequestBody, undef, 1);

        return;
    }

    Error("Adding in $Path is not implemented!");
}

sub _remove {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $ProductType = $self->{'ProductType'};
    my $AttributesFolder = $ProductType->folder('Attributes');
    Error('Attribute not exists:' . $Value->{'name'}) unless $AttributesFolder->existsChild($Value->{'name'});

    my $Attribute = $AttributesFolder->child($Value->{'name'});

    if($Path eq '/attributevalues') {
        #remove attribute values
        my $ahAttributeValues = $Value->{'values'};
        foreach my $Value (@$ahAttributeValues) {
            next unless $Attribute->folder('PreDefAttributes')->existsChild($Value->{'name'});
            $Attribute->folder('PreDefAttributes')->child($Value->{'name'})->delete;
        }
        return;
    }
    if($Path eq '/attribute') {
        # remove the whole attribute
        $Attribute->delete;

        return;
    }
    Error("Removing $Path is not implemented!");
}

1;
