#========================================================================================
# nocomments
# package      DE_EPAGES::Saferpay2::API::Constants
# state        private
#----------------------------------------------------------------------------------------
# description  constants for cartridge
#========================================================================================
package DE_EPAGES::Saferpay2::API::Constants;
use base Exporter;

use strict;



our @EXPORT_OK = qw (
    SAFERPAY_SPEC_VERSION
    INTERNAL_CARTRIDGE_NAME
    SUPPORTED_CURRENCIES
    SUPPORTED_PAYMENT_METHODS_MAP
    PAYMENT_TYPE_SAFERPAY2
    @SPLIT_GATEWAY_METHODS
    $SPLIT_GATEWAY_METHODS_MAXROWS
    SAFERPAY_PAYMENT_GROUP_PATH
    AUTHORIZED_PAYMENT_STATUS
    CAPTURED_PAYMENT_STATUS
    FAILED_PAYMENT_STATUS
    FAILED_REPEAT_PAYMENT_STATUS
    PAYMENT_METHOD_KLARNA
    REQUIRED_BILLING_ADDRESS_FIELDS_KLARNA
);


use constant SAFERPAY_SPEC_VERSION => '1.21';
use constant PAYMENT_METHOD_KLARNA => 'Klarna';

# This name is used to load objects from database (DON'T CHANGE!)
use constant INTERNAL_CARTRIDGE_NAME => 'Saferpay2';

use constant AUTHORIZED_PAYMENT_STATUS => 'AUTHORIZED';
use constant CAPTURED_PAYMENT_STATUS => 'CAPTURED';
use constant FAILED_PAYMENT_STATUS => 'FAILED';
use constant FAILED_REPEAT_PAYMENT_STATUS => 'FAILED-REPEAT';


# Fields which the merchant needs to set to mandatory in the registration settings for Klarna
use constant REQUIRED_BILLING_ADDRESS_FIELDS_KLARNA => qw ( CountryID );

# Currencies which are supported by Saferpay2.
use constant SUPPORTED_CURRENCIES => (qw(
    EUR
    CHF
));

use constant SUPPORTED_PAYMENT_METHODS_MAP => {
    ApplePay    => {
        Wallets => [ qw( APPLEPAY ) ]
    },
    BanContact => {
        PaymentMethods => [ qw( BANCONTACT ) ]
    },
    CreditDebit => {
        PaymentMethods => [ qw( AMEX BONUS DINERS JCB MAESTRO MASTERCARD UNIONPAY VISA VPAY ) ]
    },
    DirectDebit => {
        PaymentMethods => [ qw( DIRECTDEBIT ) ]
    },
    EPS => {
        PaymentMethods => [ qw( EPS ) ]
    },
    iDEAL => {
        PaymentMethods => [ qw( IDEAL ) ]
    },
    Klarna => {
        PaymentMethods => [ qw( KLARNA ) ]
    },
    PayPal => {
        PaymentMethods => [ qw( PAYPAL ) ]
    },
    Postfinance => {
        PaymentMethods => [ qw( POSTCARD POSTFINANCE ) ]
    },
    Sofort => {
        PaymentMethods => [ qw( SOFORT ) ]
    },
    TWINT => {
        PaymentMethods => [ qw( TWINT ) ]
    },
};

use constant PAYMENT_TYPE_SAFERPAY2 => 'Saferpay2';


our @AllPaymentMethods = (
    #@CreditCards
    {
        'isGroup'  => 0,
        'ID'       => 'ApplePay',
        'REAL_SGM' => 'ApplePay',
        'Name'     => 'ApplePay',
        'LogoName' => 'ApplePay2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'BanContact',
        'REAL_SGM' => 'BanContact',
        'Name'     => 'BanContact',
        'LogoName' => 'BanContact2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'CreditDebit',
        'REAL_SGM' => 'CreditDebit',
        'Name'     => 'CreditDebit',
        'LogoName' => 'CreditCard2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'DirectDebit',
        'REAL_SGM' => 'DirectDebit',
        'Name'     => 'DirectDebit',
        'LogoName' => 'DirectDebit2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'EPS',
        'REAL_SGM' => 'EPS',
        'Name'     => 'EPS',
        'LogoName' => 'EPS2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'iDEAL',
        'REAL_SGM'      => 'iDEAL',
        'Name'          => 'iDEAL',
        'LogoName'      => 'Ideal2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'Klarna',
        'REAL_SGM'      => 'Klarna',
        'Name'          => 'Klarna',
        'LogoName'      => 'Klarna2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'PayPal',
        'REAL_SGM' => 'PayPal',
        'Name'     => 'PayPal',
        'LogoName' => 'PayPal2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'Postfinance',
        'REAL_SGM' => 'Postfinance',
        'Name'     => 'Postfinance',
        'LogoName' => 'PostFinance2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'Sofort',
        'REAL_SGM' => 'Sofort',
        'Name'     => 'Sofort',
        'LogoName' => 'Sofort2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'TWINT',
        'REAL_SGM' => 'TWINT',
        'Name'     => 'TWINT',
        'LogoName' => 'Twint'
    },
);


our @SPLIT_GATEWAY_METHODS = (
    {
        'isGroup'  => 1,
        'ID'       => '_01',
        'Name'     => '',
        'GrpCount' => scalar(@AllPaymentMethods)
    },
    @AllPaymentMethods
);

our $SPLIT_GATEWAY_METHODS_MAXROWS = scalar(@AllPaymentMethods);


use constant SAFERPAY_PAYMENT_GROUP_PATH => '/PaymentTypes/Saferpay2/PaymentTypeMethods/SaferpayPaymentGroup/';

1;