#========================================================================================
# package      DE_EPAGES::Zettle::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Zettle::API::Cartridge;
use base DE_EPAGES::Design::API::DesignInstaller;

use strict;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Zettle::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Zettle.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Zettle',
        'Version'            => '7.85.0',
        'Patches'            => ['7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0','7.81.0','7.82.0','7.83.0','7.84.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_7_80_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_7_80_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '7.80.0' to version '7.81.0'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '7.81.0' | string
#========================================================================================
sub patchDBStructure_7_80_0 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

    }

    $self->importPatchDir('Patch_7_80_0', 'PatchDBStructure');

    return '7.81.0';
}

1;
